from dagster_mcp.server import list_code_locations, list_backfills, get_instance_status


class TestListCodeLocations:
    def test_locations(self, mock_gql):
        mock_gql({"data": {"workspaceOrError": {"locationEntries": [
            {"name": "loc1", "loadStatus": "LOADED",
             "locationOrLoadError": {
                 "name": "loc1", "repositories": [{"name": "repo1"}]}},
        ]}}})
        result = list_code_locations()
        assert len(result) == 1
        assert result[0]["name"] == "loc1"

    def test_empty(self, mock_gql):
        mock_gql({"data": {"workspaceOrError": {"locationEntries": []}}})
        assert list_code_locations() == []


class TestListBackfills:
    def test_backfills(self, mock_gql):
        mock_gql({"data": {"partitionBackfillsOrError": {"results": [
            {"backfillId": "bf1", "status": "COMPLETED",
             "numPartitions": 10, "timestamp": "1000",
             "partitionNames": ["p1", "p2"],
             "partitionSetName": "my_set"},
        ]}}})
        result = list_backfills()
        assert len(result) == 1
        assert result[0]["backfillId"] == "bf1"

    def test_empty(self, mock_gql):
        mock_gql({"data": {"partitionBackfillsOrError": {"results": []}}})
        assert list_backfills() == []

    def test_custom_limit(self, mock_gql):
        mock_post = mock_gql({"data": {"partitionBackfillsOrError": {"results": []}}})
        list_backfills(limit=5)
        payload = mock_post.call_args.kwargs["json"]
        assert payload["variables"]["limit"] == 5


def _instance_response(daemons=None, queued_runs=None, locations=None):
    return {"data": {
        "instance": {"daemonHealth": {"allDaemonStatuses": daemons or []}},
        "runsOrError": {"results": queued_runs or []},
        "workspaceOrError": {"locationEntries": locations or []},
    }}


class TestGetInstanceStatus:
    def test_healthy(self, mock_gql):
        mock_gql(_instance_response(
            daemons=[
                {"daemonType": "SCHEDULER", "required": True,
                 "healthy": True, "lastHeartbeatTime": "1000"},
                {"daemonType": "SENSOR", "required": True,
                 "healthy": True, "lastHeartbeatTime": "1000"},
            ],
            locations=[
                {"name": "loc1", "loadStatus": "LOADED",
                 "locationOrLoadError": {"name": "loc1"}},
            ],
        ))
        result = get_instance_status()
        assert result["healthy"] is True
        assert len(result["daemons"]) == 2
        assert result["queued_runs_count"] == 0
        assert result["code_location_errors"] == []

    def test_unhealthy_daemon(self, mock_gql):
        mock_gql(_instance_response(
            daemons=[
                {"daemonType": "SCHEDULER", "required": True,
                 "healthy": False, "lastHeartbeatTime": "500"},
            ],
        ))
        result = get_instance_status()
        assert result["healthy"] is False
        assert result["daemons"][0]["healthy"] is False

    def test_code_location_error(self, mock_gql):
        mock_gql(_instance_response(
            daemons=[
                {"daemonType": "SCHEDULER", "required": True,
                 "healthy": True, "lastHeartbeatTime": "1000"},
            ],
            locations=[
                {"name": "broken_loc", "loadStatus": "LOADED",
                 "locationOrLoadError": {"message": "ImportError: no module"}},
            ],
        ))
        result = get_instance_status()
        assert result["healthy"] is False
        assert len(result["code_location_errors"]) == 1
        assert result["code_location_errors"][0]["name"] == "broken_loc"

    def test_queued_runs(self, mock_gql):
        mock_gql(_instance_response(
            daemons=[
                {"daemonType": "SCHEDULER", "required": True,
                 "healthy": True, "lastHeartbeatTime": "1000"},
            ],
            queued_runs=[{"runId": "r1"}, {"runId": "r2"}, {"runId": "r3"}],
        ))
        result = get_instance_status()
        assert result["queued_runs_count"] == 3
        assert result["healthy"] is True



import pytest
from unittest.mock import MagicMock

import httpx

from dagster_mcp.server import gql


class TestGql:
    def test_successful_query(self, mock_gql):
        mock_gql({"data": {"runsOrError": {"results": []}}})
        result = gql("query { runsOrError { results { runId } } }")
        assert result == {"runsOrError": {"results": []}}

    def test_connect_error(self, monkeypatch):
        monkeypatch.setattr(
            httpx, "post", MagicMock(side_effect=httpx.ConnectError("refused"))
        )
        with pytest.raises(RuntimeError, match="Cannot connect to Dagster"):
            gql("query { }")

    def test_timeout_error(self, monkeypatch):
        monkeypatch.setattr(
            httpx, "post", MagicMock(side_effect=httpx.TimeoutException("timed out"))
        )
        with pytest.raises(RuntimeError, match="timed out after 30s"):
            gql("query { }")

    def test_http_error(self, mock_gql):
        mock_gql({"error": "Internal Server Error"}, status_code=500)
        with pytest.raises(RuntimeError, match="HTTP 500"):
            gql("query { }")

    def test_graphql_errors(self, mock_gql):
        mock_gql({
            "errors": [{"message": "Field 'foo' not found"}],
            "data": None,
        })
        with pytest.raises(RuntimeError, match="GraphQL error.*Field 'foo' not found"):
            gql("query { foo }")

    def test_passes_variables(self, mock_gql):
        mock_post = mock_gql({"data": {"result": "ok"}})
        gql("query Q($id: ID!) { result }", {"id": "123"})
        call_kwargs = mock_post.call_args
        payload = call_kwargs.kwargs.get("json") or call_kwargs[1].get("json")
        assert payload["variables"] == {"id": "123"}

    def test_empty_variables_default(self, mock_gql):
        mock_post = mock_gql({"data": {"result": "ok"}})
        gql("query { result }")
        call_kwargs = mock_post.call_args
        payload = call_kwargs.kwargs.get("json") or call_kwargs[1].get("json")
        assert payload["variables"] == {}



import pytest

from dagster_mcp import server


class TestBuildHeaders:
    def test_no_token(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "")
        monkeypatch.setattr(server, "DAGSTER_EXTRA_HEADERS", "")
        headers = server._build_headers()
        assert headers == {}

    def test_with_token(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "my-token")
        monkeypatch.setattr(server, "DAGSTER_EXTRA_HEADERS", "")
        headers = server._build_headers()
        assert headers == {"Dagster-Cloud-Api-Token": "my-token"}

    def test_extra_headers_valid(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "")
        monkeypatch.setattr(
            server, "DAGSTER_EXTRA_HEADERS",
            '{"Authorization": "Bearer abc", "X-Custom": "val"}'
        )
        headers = server._build_headers()
        assert headers == {"Authorization": "Bearer abc", "X-Custom": "val"}

    def test_extra_headers_merged_with_token(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "my-token")
        monkeypatch.setattr(
            server, "DAGSTER_EXTRA_HEADERS",
            '{"X-Custom": "val"}'
        )
        headers = server._build_headers()
        assert headers == {
            "Dagster-Cloud-Api-Token": "my-token",
            "X-Custom": "val",
        }

    def test_extra_headers_malformed_json(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "")
        monkeypatch.setattr(server, "DAGSTER_EXTRA_HEADERS", "{not valid json}")
        with pytest.raises(RuntimeError, match="valid JSON object"):
            server._build_headers()

    def test_extra_headers_not_a_dict(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "")
        monkeypatch.setattr(server, "DAGSTER_EXTRA_HEADERS", '["a", "b"]')
        with pytest.raises(RuntimeError, match="must be a JSON object"):
            server._build_headers()

    def test_extra_headers_non_string_values(self, monkeypatch):
        monkeypatch.setattr(server, "DAGSTER_API_TOKEN", "")
        monkeypatch.setattr(server, "DAGSTER_EXTRA_HEADERS", '{"key": 123}')
        with pytest.raises(RuntimeError, match="must be strings"):
            server._build_headers()
